#include <mpi.h>
#include <stdio.h>

double f1(double x){
	return 1/(1+x*x);
}

//h is used only approximately
double midpoint(double lb,double ilength, double h){
	int numsteps = ilength/h;
	int i;
	double res;
	h = ilength/numsteps;
	for(i=0;i<numsteps;i++){
		res += h*f1(lb + h*i + h/2);
	}
	return res;	
}

int main(int argc, char** argv){
	/* our infinity is quite a bit finite.*/
	double infty = 1000000.0;
	int rank,size,i;
	double ilength,h=0.001;
	double tres,res,start;

	MPI_Init(&argc,&argv);
	MPI_Comm_size(MPI_COMM_WORLD,&size);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);

	start = MPI_Wtime();

	ilength = 2*infty/size;
	tres = midpoint(-infty+rank*ilength,ilength,h);
	printf("Zwischenzeit Prozess %d: %f\n",rank,-start+MPI_Wtime());

	//send the result
	if(rank != 0){
		MPI_Send(&tres,1,MPI_DOUBLE,0,0,MPI_COMM_WORLD);
	} else {
	//Master Process
		res = tres;
		printf("Process %d result: %f\n",0,tres);
		for(i=1;i<size;i++){
			MPI_Recv(&tres,1,MPI_DOUBLE,i,0,MPI_COMM_WORLD,NULL);
			printf("Process %d result: %f\n",i,tres);
			res += tres;
		}
		printf("Total result: %1.10f\n",res);
		start -= MPI_Wtime();
		printf("Time: %3.10f\n",-start);
	}

	MPI_Finalize();

	return 0;
}	
